"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stringify_1 = require("@code-engine/stringify");
const utils_1 = require("@code-engine/utils");
const ono_1 = require("ono");
const clone_context_1 = require("../clone/clone-context");
const clone_file_1 = require("../clone/clone-file");
const messenger_1 = require("../worker-thread/messenger");
/**
 * Executes commands in a worker thread that are sent by a corresponding `Worker` running on the main thread.
 * @internal
 */
class Executor extends messenger_1.Messenger {
    constructor(threadId, port) {
        super(port);
        this._processors = new Map();
        this.threadId = threadId;
    }
    /**
     * Imports the specified `FileProcessor` module.
     */
    async importFileProcessor(message) {
        let { moduleUID, moduleId, cwd } = message;
        let fileProcessor;
        try {
            // Import the plugin module
            let exports = await utils_1.importModule(moduleId, cwd);
            let defaultExport = exports.default;
            if (defaultExport === undefined || defaultExport === null) {
                throw ono_1.ono.type(`CodeEngine plugin modules must export a function.`);
            }
            else if (typeof defaultExport !== "function") {
                throw ono_1.ono.type(`The module exported ${stringify_1.stringify(defaultExport, { article: true })}. ` +
                    `CodeEngine plugin modules must export a function.`);
            }
            // This could be a FileProcessor or a FactoryFunction
            if (message.data === undefined) {
                // The exported function is a FileProcessor
                fileProcessor = defaultExport;
            }
            else {
                // The exported function is a FactoryFunction, so call the factory with the given data.
                let factory = defaultExport;
                let product = await Promise.resolve(factory(message.data));
                if (product === undefined || product === null) {
                    throw ono_1.ono.type(`The ${factory.name || "exported"} function must return a CodeEngine file processor.`);
                }
                else if (typeof product !== "function") {
                    throw ono_1.ono.type(`The ${factory.name || "exported"} function returned ${stringify_1.stringify(product, { article: true })}. ` +
                        "Expected a CodeEngine file processor.");
                }
                // The factory produced a FileProcessor
                fileProcessor = product;
            }
            // Store the FileProcessor so we can call it later
            this._processors.set(moduleUID, fileProcessor);
            // Reply with information about the module
            this.postReply({ to: message.id, type: "fileProcessorImported", name: fileProcessor.name });
        }
        catch (error) {
            throw utils_1.typedOno(error, { workerId: this.threadId, moduleId }, `Error importing module: ${moduleId}`);
        }
    }
    /**
     * Imports the specified JavaScript module.
     */
    async importModule(message) {
        let { moduleId, cwd } = message;
        try {
            // Import the plugin module
            let exports = await utils_1.importModule(moduleId, cwd);
            let defaultExport = exports.default || exports;
            if (typeof defaultExport === "function") {
                // Call the exported function with the given data
                let factory = defaultExport;
                await Promise.resolve(factory(message.data));
            }
            // Reply that we're done importing the module
            this.postReply({ to: message.id, type: "finished" });
        }
        catch (error) {
            throw utils_1.typedOno(error, { workerId: this.threadId, moduleId }, `Error importing module: ${moduleId}`);
        }
    }
    /**
     * Processes a file using the specified plugin.
     */
    async processFile(message) {
        // Create clones of the File and Context
        let file = utils_1.createFile(message.file);
        let context = clone_context_1.createContext(this, message.id, message.context);
        // Process the file using the specified plugin
        let fileProcessor = this._processors.get(message.moduleUID);
        let output = await fileProcessor.call(undefined, file, context);
        for await (let fileInfo of utils_1.iterate(output)) {
            let outFile = utils_1.normalizeFileInfo(fileInfo);
            let [outFileClone, transferList] = clone_file_1.cloneFile(outFile);
            this.postReply({ to: message.id, type: "file", file: outFileClone }, transferList);
        }
        // Let the worker know that we're done yielding files
        this.postReply({ to: message.id, type: "finished" });
    }
}
exports.Executor = Executor;
//# sourceMappingURL=executor.js.map